// ======================================================== IBJdepo.c
//
// Deposition parameters according to VDI 3782 Part 5 (draft)
// ==========================================================
//
// Copyright (C) Janicke Consulting, 26427 Dunum, Germany, 2004
// email: info@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// 1.2.2  uj  2003-06-23 Parameter Da entfernt
// 1.2.3  uj  2003-07-04 Ausgabe in eine Datei
// 1.2.5  uj  2003-07-22 default D0=6*Z0, output modified
// 1.2.6  uj  2003-08-07 washout rate modified
// 
//====================================================================

#include <stdio.h>
#include <string.h>
#include <math.h>

#define  VERSION   "1.2.6"

#define  KBOLTZ    1.380658e-23
#define  RGAS      0.0821
#define  GRAV      9.81
#define  AIRWEIGHT 28.9644
#define  PI        3.141592654
#define  L_PRT_FAC 4.4e-4
#define  L_PRT_EXP 0.8
#define  L_PRT_RED 0.95
#define  L_PRT_DIA 10.e-6
#define  TZERO     273.15
#define  FA        5
#define  FB        67


char  *argv[10];
float ClcVsed(float, float);
float ClcLambda(float);
float ClcVdep(void);
float GetSlipC(float);
void  PrintResult(FILE *);
float GetDifAerosol(float, float, float);
float GetDifGas(float);
float GetEta( float );
float GetHm(float, float, float, int);
float GetLm(float, int);
int   GetClass(float, float);
float GetU(float, float, float, float, float);
float GetVs(float, float);
void  dowork1(void);
void  dowork2(void);
void  dowork3(float);
void  dowork4(void);
void  dowork5(void);
void  Help(void);
void  ErrorMsg(char *);

static float Nb, Nre, Nsch, Nst, Hm, Lm;
static float Vd, Vdb, Vs, Dif, Ra, Rb, Rc, dif;
static float Zr, Ua, Ha, D0, Z0, Ust, D, Rho;
static int nz0TAL=9, ClassIndex=2;
static char Out[256];
static char Class[10];
static float z0TAL[9] = { 0.01, 0.02, 0.05, 0.10, 0.20, 0.50, 1.0, 1.5, 2.0 };
static float lmTAL[6][9] =
               {   7,     9,   13,   17,   24,   40,   65,   90,  118,
                  25,    31,   44,   60,   83,  139,  223,  310,  406,
               99999, 99999,99999,99999,99999,99999,99999,99999,99999,
                 -25,   -32,  -45,  -60,  -81, -130, -196, -260, -326,
                 -10,   -13,  -19,  -25,  -34,  -55,  -83, -110, -137,
                  -4,    -5,   -7,  -10,  -14,  -22,  -34,  -45,  -56 };
static char ClassNames[6][10] = { "I", "II", "III/1", "III/2", "IV", "V" };

static float Nkfa = 43;
static float kfa_jo81[43][6] = {

// vd (cm/s), dev, d (mum), ust (cm/s), z0 (cm), no

 0.024,  11,  0.4,  56.7,  19.9, 38,
 0.013,   8,  0.8,  21.7,   0.4, 29,
 0.011,  11,  1.1,  28.3,   2.5, 27,
 0.011,  13,  1.4,  20.6,   2.7, 26,
 0.034,  27,  1.4,  18.8,   2.5, 50,
 0.023,   9,  2.0,  21.4,   4.5, 17,
 0.025,   8,  2.0,  21.2,   2.2, 21,
 0.039,   5,  2.9,  22.6,   0.8, 18,
 0.052,   8,  2.9,  22.6,   0.8, 18,
 0.056,  11,  3.1,  23.6,   2.5, 22,
 0.055,   9,  3.1,  23.6,   2.5, 22,
 0.041,   8,  3.1,  26.0,   0.4, 24,
 0.040,  10,  3.1,  30.4,   2.1, 31,
 0.018,   9,  3.1,  19.7,   4.3, 32,
 0.013,   8,  3.1,  13.3,   6.4, 33,
 0.017,   7,  3.1,  21.5,   28., 34,
 0.029,   6,  3.1,  32.6,   3.5, 35,
 0.042,   7,  3.1,  24.4,   6.6, 39,
 0.025,   9,  3.1,  12.7,   1.8, 40,
 0.035,  14,  3.1,  34.6,   0.6, 42,
 0.078,   0,  3.1,  23.6,   2.5, 22,
 0.054,   0,  3.1,  23.6,   2.5, 22,
 0.012,  15,  3.2,  31.8,   5.5, 15,
 0.034,  10,  3.3,  17.7,   0.4, 25,
 0.036,  13,  3.3,  39.9,   3.0, 28,
 0.049,  21,  3.4,  22.9,   1.6, 14,
 0.25,   18,  3.7,  21.2,   0.8, 45,
 0.36,   16,  4.3,  27.7,   2.8, 46,
 0.39,   16,  4.3,  23.5,   2.0, 44,
 0.320,  21,  4.6,   5.7,   0.3, 19,
 0.049,  26,  4.6,  14.7,   0.1, 12,
 0.380,  21,  5.2,  24.3,   0.6, 20,
 0.340,  15,  5.2,  24.3,   0.6, 20,
 0.17,   11,  5.2,  13.6,   1.1, 48,
 0.43,   22,  5.4,  27.0,   3.1, 47,
 0.39,   15,  5.4,  27.0,   3.1, 47,
 0.76,   13,  5.7,  19.3,   1.0, 52,
 1.090,  16,  7.1,  18.1,   1.8, 37,
 3.58,    0,  7.1,  22.9,   1.0, 51,
 2.16,    0,  8.9,  21.8,   8.6, 36,
 8.12,   12, 14.4,  32.9,   7.5, 41,
 6.31,   35, 16.8,  22.1,   7.3, 49,
 4.52,   23, 17.1,  20.8,   1.3, 43
 };
 


int main( int argc, char *argv[] ) {
  char *ss;
  int i, work;
  float iprec;
  FILE *fp;
  if (argc < 2) Help();
  ClassIndex = 2;
  strcpy(Class, ClassNames[ClassIndex]);
  *Out = 0;
  iprec = 1.;
  Zr = 1.5;
  Ua = 3.0;
  Ha = 10.0;
  Z0 = 0.5;
  D0 = 6*Z0;
  Lm = 0.;
  Hm = 0.;
  Ust = 0.;
  D = 10.e-6;
  Rho = 1000;
  Ra = -1;
  Rb = -1;
  Dif = -1;
  Rc = 0;
  iprec = 0;
  work = -1;
  for (i=1; i<argc; i++) {  
    ss = argv[i];
    if (*ss == '-') {
      switch(ss[1]) {
        case 'l':  sscanf(ss+2, "%f", &Lm);
                   break;
        case 'c':  sscanf(ss+2, "%s", Class);
                   break;
        case 'C':  sscanf(ss+2, "%f", &Rc);
                   break;
        case 'D':  sscanf(ss+2, "%f", &Dif);
                   break;
        case 'm':  sscanf(ss+2, "%f", &Hm);
                   break;
        case 'o':  sscanf(ss+2, "%s", &Out);
                   break;
        case 'z':  D0 = -1;
                   Z0 = -1;
                   sscanf(ss+2, "%f,%f", &Z0, &D0);
                   if (Z0 <= 0) Z0 = 0.5;
                   if (D0 < 0)  D0 = 6*Z0;
                   break;
        case 'u':  sscanf(ss+2, "%f,%f", &Ua, &Ha);                
                   break;
        case 'r':  sscanf(ss+2, "%f", &Zr);
                   break;  
        case 'd':  Rho = 1.;
                   sscanf(ss+2, "%f,%f", &D, &Rho);
                   D /= 1.e6; 
                   Rho *= 1000.;
                   break;  
        case 's':  sscanf(ss+2, "%f", &Ust);
                   break; 
        case 'W':  sscanf(ss+2, "%d", &work);
                   break; 
        case 'I':  sscanf(ss+2, "%f", &iprec);
                   break;                   
        case 'h':  Help();
                   break;
        default:   printf("*** Unbekannte Aufrufoption %s\n", ss);
                   exit(-99);
      }  
    }
  } 
  if (Z0 < 0) Z0 = 0.5;
  if (D0 < 0) D0 = 0;
  
  //////////////////
  if (work == 1) { dowork1(); exit(0); }
  if (work == 2) { dowork2(); exit(0); }
  if (work == 3) { dowork3(iprec); exit(0); }
  if (work == 4) { dowork4(); exit(0); }
  if (work == 5) { dowork5(); exit(0); }
  //////////////////
  
  if (Z0<=0. || Ua<=0. || Ha <=0. || D<0.) Help();
  
  if (Out) fp = fopen(Out, "w");
  if (!fp) fp = stdout;
  fprintf(fp, "IBJdepo %s (C) Ing.-Buero Janicke 2002-2004\n", VERSION);
  fprintf(fp, "IBJdepo comes with ABSOLUTELY NO WARRANTY\n");
  fprintf(fp, "IBJdepo is free software under the GNU PUBLIC LICENCE\n\n");
  fprintf(fp, "Depositionsparameter nach VDI 3782/5 (Entwurf)\n\n");
  if (iprec > 0) {
    if (D <= 0.)
      ErrorMsg("Auswaschrate kann nur fuer Aerosole (D > 0) bestimmt werden.");
    else {
      fprintf(fp, "- Aerosol D=%5.2f m  I=%5.2f mm/h\n\n", 
      D*1.e6, iprec);
      fprintf(fp, "ll= %1.1e 1/s (Auswaschrate)\n", ClcLambda(iprec));
    }
    exit(0);
  }
  
  ClcVdep();  
  PrintResult( fp );
  exit(0);
}

//--------------------------------------------------------------- printResult
void PrintResult(FILE *fp) {
  if (D > 0.) 
    fprintf(fp, "- Aerosol\n");
  else
    fprintf(fp, "- Gas\n");
  fprintf(fp, "- D   =%7.3f um      z0 =%6.3f m \n",
    (D > 0) ? D*1.e6 : 0, Z0);
  fprintf(fp, "- rho =%7.3f g/cm3   d0 =%6.3f m    Hm=%6.0f m\n",
    (D > 0) ? Rho/1000 : 0, D0, Hm);
  fprintf(fp, "- Nst =%9.1e       ust=%6.2f m/s  Lm=%6.0f m\n",
    (D > 0) ? Nst : 0, Ust, Lm);
  fprintf(fp, "- Nsch=%9.1e       ua =%6.1f m/s  ur=%6.1f m/s\n",
    Nsch, Ua, GetU(Zr, Z0, D0, Lm, Ust)); 
  fprintf(fp, "- Dth =%9.1e m2/s  ha =%6.1f m    zr=%6.1f m\n",
    dif, Ha, Zr);
  fprintf(fp, "\n");
  fprintf(fp, "ra= %7.3f s/cm (Aerodynamischer Widerstand)\n", Ra/100);
  fprintf(fp, "rb= %7.3f s/cm (Quasi-laminarer Widerstand)\n", Rb/100);
  fprintf(fp, "rc= %7.3f s/cm (Oberflaechen-Widerstand)\n", Rc/100);
  fprintf(fp, "vs= %7.3f cm/s (Sinkgeschwindigkeit)\n", Vs*100);
  fprintf(fp, "vd= %7.3f cm/s (Depositionsgeschwindigkeit)\n", Vd*100);
}  


//--------------------------------------------------------------- ErrorMsg
void ErrorMsg(char *s) {
  printf("*** Error: %s\n", s);
  exit(-99);
} 
   
//------------------------------------------------------------------------ Help
void Help() {
  printf("IBJdepo %s (c) Ing.-Buero Janicke 2002-2004\n", VERSION);
  printf("IBJdepo comes with ABSOLUTELY NO WARRANTY\n");
  printf("IBJdepo is free software under the GNU PUBLIC LICENCE\n\n");
  printf("Depositionsparameter nach VDI 3782/5 (Entwurf)\n");
  printf("Aufruf  : IBJdepo <Option> <Option> ...\n");
  printf("Optionen:\n");
  printf("    -c<Class>     Klug/Manier-Klasse (I, II, III/1, ...)   [III/1]\n");
  printf("    -d<D>[,Rho>]  Aerosol-Durchmesser (um), Dichte (g/cm3) [10,1]\n");
  printf("    -l<Lm>        Monin-Obukhov-Laenge (m)                 [99999]\n");
  printf("    -m<Hm>        Mischungsschichthoehe (m)                []\n");
  printf("    -s<Ust>       Schubspannungsgeschwindigkeit (m/s)      []\n");
  printf("    -r<Zr>        Referenzhoehe fuer Ra (m)                [1.5]\n");
  printf("    -u<Ua,Ha>     Windgeschw. Ua (m/s) in Hoehe Ha (m)     [3,10]\n");
  printf("    -z<Z0>[,<D0>] Rauhigkeitslaenge (m), Verdr.hoehe (m)   [0.5,6*Z0]\n");  
  printf("    -C<Rc>        Oberflaechen-Widerstand (s/m)            [0]\n");
  printf("    -D<Dif>       Diffusivitaet (m2/s)                     []\n");
  printf("    -I<Int>       Berechne Auswaschrate fuer Aerosole\n");
  printf("                  mit Regenintensitaet <Int> (mm/h)        [1]\n");
  printf("    -W<n>         Datensaetze zu den Abbildungen in VDI 3782/5 (Entwurf):\n");
  printf("                  <n>=1: vd(D)\n");
  printf("                  <n>=2: vd(Ust)\n");
  printf("                  <n>=3: Lambda(D)\n");
  printf("                  <n>=4: Massenverteilung\n");
  printf("                  <n>=5: Juelich-Experimente\n");
  exit(0);  
}

//-------------------------------------------------------------------- ClcLambda
/** Returns an approximate value for the washout rate for aerosols
 *  @param iprec  precipitation (mm/h)
 *  @return washout rate (1/s)
 */
float ClcLambda( float iprec ) {
  float e;
  if (D <= 0.) return -999.;
  if (iprec <= 0.) return 0.;
  e = 1. - L_PRT_RED * exp(-D*D/(L_PRT_DIA*L_PRT_DIA));
  return e * L_PRT_FAC * pow(iprec, L_PRT_EXP);
}


//-------------------------------------------------------------------- ClcVdep
/** Returns the deposition velocity and different auxiliary 
 *  parameters and stores them into global variables.
 *  @return deposition velocity (m/s)
 */
float ClcVdep() {
  int i;
  float tmp, rhoair, eta;
  tmp = TZERO + 15.;
  eta = GetEta( tmp );
  rhoair = AIRWEIGHT/(RGAS*tmp);
  for (i=0; i<6; i++) 
    if (0 == strcmp(Class, ClassNames[i])) break;
  if (i > 5) i = 2;
  ClassIndex = i;
  if (Lm == 0.) 
    Lm = GetLm( Z0, ClassIndex );
  else 
    ClassIndex = GetClass( Z0, Lm );
  if (Ust == 0.) 
    Ust = Ua/GetU( Ha, Z0, D0, Lm, 1. );
  else 
    Ua = GetU( Ha, Z0, D0, Lm, Ust );
  if (Hm == 0.) 
    Hm = GetHm( Z0, Lm, Ust, ClassIndex );
  Ra = (GetU( Zr, Z0, D0, Lm, Ust ) - GetU( Z0, Z0, D0, Lm, Ust ))/(Ust*Ust);
  if (D > 0.) { // aerosols
    Vs = GetVs( D, Rho );
    if (Dif <= 0) dif  = GetDifAerosol( D, tmp, eta );
    else          dif = Dif;
    Nsch = eta / (rhoair * dif);
    Nst  = Vs * Ust * Ust * rhoair / (eta * GRAV);
    Rb = 1. / (Ust*(pow(Nsch, -0.6666) + Nst*Nst/(FB+Nst*Nst)));    
    Vd =  Vs / (1. - exp(-(Ra + Rb + Rc) * Vs));
    Vdb = Vs / (1. - exp(-(Rb + Rc) * Vs));
  }
  else {
    if (Dif <= 0) dif = GetDifGas( tmp );
    else          dif = Dif;
    Nsch = eta / (rhoair * dif);
    Rb = FA * pow(Nsch, 0.6666) / Ust;
    Vs = 0.;
    Vd = 1. / (Ra + Rb + Rc);
    Vdb = 1. / (Rb + Rc);
  }   
  return Vd;
}

//------------------------------------------------------------------- GetDa
/** Returns the air viscosity
 *  @param t   tmp (K)
 *  @return    air viscosity
 */
float GetEta( float tmp ) {
  float eta;
  eta = 1.e-4 * (0.1718 + 0.00049*(tmp-TZERO));
  return eta;
}

//------------------------------------------------------------------- GetLm
/** Returns the Monin-Obukhov length according to TA Luft 2002. 
 *  @param z0  roughness length (m)
 *  @param ci  stability class index (0..5)
 *  @return Monin-Obukhov length (m)
 */
float GetLm( float z0, int ci ) {
  int k, kk;
  float z1, z2, f1, f2, lm;
  kk = ci;
  if (ci<0 || ci>5) kk = 2;
  if (z0 <= z0TAL[0]) lm = lmTAL[kk][0];
  else {
    for (k=1; k<nz0TAL; k++)
      if (z0>z0TAL[k-1] && z0<=z0TAL[k]) break;
    if (k > nz0TAL-1) lm = lmTAL[kk][nz0TAL-1];
    z1 = log(z0TAL[k-1]);
    z2 = log(z0TAL[k]);
    f1 = 1./lmTAL[kk][k-1];
    f2 = 1./lmTAL[kk][k];
    lm = 1./(f1 + (log(z0)-z1)/(z2-z1)*(f2-f1));    
  }
  return lm;
}

//------------------------------------------------------------------- GetClass
/** Returns the stability class according to TA Luft 2002.
 *  @param z0  roughness length (m)
 *  @param lm  Monin-Obukhov length (m)
 *  @return stability class index (0..5)
 */
int GetClass(float z0, float lm) {
  int k, kk;
  float f1, f2;
  if (z0 <= z0TAL[0]) k = 0;
  else if (z0 > z0TAL[nz0TAL-1]) k = nz0TAL-1;
  else
    for (k=1; k<nz0TAL; k++)
      if (z0>z0TAL[k-1] && z0<=z0TAL[k]) break;
  if (lm == 0) kk = 2;
  else {
    f1 = 1./lm;
    for (kk=5; kk>0; kk--) {
      f2 = (lmTAL[kk][k]+lmTAL[kk-1][k])/(2*lmTAL[kk][k]*lmTAL[kk-1][k]);
      if (f1 < f2) break;
    }
  }
  return kk;
}

//------------------------------------------------------------------- GetHm
/** Returns the mixing height according to TA Luft 2002.
 *  @param z0  roughness length (m)
 *  @param lm  Monin-Obukhov length (m)
 *  @param ust friction velocity (m/s)
 *  @param ci  stability class index (0..5)
 *  @return mixing height (m)
*/
float GetHm(float z0, float lm, float ust, int ci) {
  int kk;
  float a, hm;
  kk = ci;
  if (kk < 0 ||kk > 5) kk = 2;
  if (lm == 0.) lm = lmTAL[kk][0];
  if (lm > 0) {
    hm = 0.3 * ust/1.e-4;
    a = sqrt(1.e-4 * lm/ust);
    if (a < 1)  hm *= a;
    if (hm > 800)  hm = 800;   
  }
  else if (kk == 3)
    hm = 800.;
  else
    hm = 1100.;
  return hm;
}
  
//--------------------------------------------------------------- getSlipC
/** Returns the slip correction (Cunningham) factor.
 *  @param d  particle diameter (m)
 *  @return   slip correction factor
 */
float GetSlipC(float d) {
  int i;
  float c;
  float b[16] =
  { 0.001, 0.002, 0.005, 
    0.01,  0.02,  0.05, 
    0.1,   0.2,   0.5, 
    1.0,   2.0,   5.0, 
    10.0,  20.0,  50.0, 
    100.0 };
  float t[16] =
  { 216.,  108.,  43.6, 
    22.2,  11.4,  4.95, 
    2.85,  1.865, 1.326, 
    1.164, 1.082, 1.032,
    1.061, 1.008, 1.003,
    1.0016 };
  d *= 1.e+6;
  c = 0.0;
  for (i=0; i<15; i++) {
    if (d >= b[i] && d < b[i+1]) {
      c = t[i+1] + (b[i+1]-d)/(b[i+1]-b[i])*(t[i] - t[i+1]);
      break;
    }
  }
  if (c == 0.0) {
    if (d <= b[0]) c = t[0];
    else           c = t[15];
  }
  return c;
}

//------------------------------------------------------------------- GetLm
/** Returns the thermal diffusivity of gases (Pruppacher 13-3)
 *  @param tmp temperature (K)
 *  @return diffusivity (m2/s)
 */
float GetDifGas( float tmp ) {
  float d = 0.211 * pow(tmp/TZERO, 1.94) * 1.e-4;
  return d;
}

//----------------------------------------------------------------- getDif
/** Returns the thermal aerosol diffusivity.
 *  @param d   aerosol diameter (m)
 *  @param tmp temperature (K)
 *  @param eta air viscosity (kg/ms)
 *  @return   diffusivity in m^2/s
 */
float GetDifAerosol( float d, float tmp, float eta ) {
  float c = GetSlipC(d);
  double df;
  df = (KBOLTZ * tmp * c)/(3 * PI * eta * d);
  return (float)df;
}


//--------------------------------------------------------------------- GetU
/** Returns the wind velocity according to TA Luft 2002
 *  @param h   height above ground (m)
 *  @param z0  roughness length (m)
 *  @param d0  displacement length (m)
 *  @param lm  Monin-Obukhov length (m)
 *  @param ust friction velocity (m/s)
 *  @return    wind velocity (m/s)
 */
float GetU(float h, float z0, float d0, float lm, float ust) {
  float zeta, zeta0, psi, x1, x2, x3, x, x0, u, hh, z, fac=1.;
  float kappa = 0.4;
  hh = 6 * z0 + d0;
  if (d0 < 0)
    d0 = 6 * z0;
  hh = 6 * z0 + d0;
  z = (h < hh) ? hh - d0 : h - d0;
  if (z < 0)
    z = 0;
  zeta = (z+z0)/lm;
  zeta0 = z0/lm;
  if (lm > 0.) {
    x1 = -5*zeta0;
    x2 = 7*log(2.) - log(zeta0) - 5*zeta0 - 4;
    x3 = 8*log(20) - 11.165 - log(2.) - log(zeta0) - 5*zeta0;
    if (zeta < 0.5)
      psi = log(zeta/zeta0) + 5*zeta + x1;
    else if (zeta <= 10.)
      psi = 8*log(zeta) + 4.25/zeta - 0.5/zeta/zeta + x2;
    else
      psi = 0.7585*zeta + x3;
    u = ust/kappa*psi;
  }
  else {
    x = pow(1 - 15*zeta, 0.25);
    x0 = pow(1 - 15*zeta0, 0.25);
    x1 = (1+x)*(1+x)/(1+x0)/(1+x0) * (1+x*x)/(1+x0*x0);
    psi = log(x1) - 2*(atan(x) - atan(x0));
    u = ust/kappa*(log(zeta/zeta0) - psi);
  }
  if (h < hh)
    u *= h / hh;
  if (u < 0)
    u = 0.;
  return u;
}

//--------------------------------------------------------------------- GetVs
/** Returns the settling velocity according to VDI 3782/1 with 
 *  Cunningham correction
 *  @param d     aerosol diameter (m)
 *  @param rho   aerosol density (kg/m3)
 *  @return settling velocity (m/s)
 */
float GetVs(float d, float rho ) {
  float dm, rm, w, logw, re, a, vg;
  static float aa[7] = {
    -0.318657e1, 
     0.992696e0, 
    -0.153193e-2, 
    -0.987059e-3, 
    -0.578878e-3, 
     0.855176e-4,
    -0.327815e-5 };
  int i;
  dm = d * 1.e+6;
  rm = rho * 1.e-3;
  w = 4.991e-5 * dm*dm*dm * rm;
  if (w < 0.24)  re = w/24;
  else {
    logw = log(w);
    a = aa[6];
    for (i=5; i>=0; i--)  a = aa[i] + a*logw;
    re = exp(a);
  } 
  vg = 1462. * re/dm; 
  Nb = w;
  Nre = re;
  return vg * 0.01 * GetSlipC( d );
}

//================================================================
// utility functions for graphic plots
//================================================================

// deposition velocity as function of diameter
void dowork1() {
  int i, j;
  float za[4] = { 0.01, 0.1, 0.5, 1.5 };
  float ua[4] = { 2, 3, 4, 5 };
  char *ca[4] = { "III/1", "III/1", "III/1", "III/1" };
  float p;
  for (i=0; i<4; i++) {
    Z0 = za[i];
    D0 = 6*Z0;
    Zr = 6*Z0;
    Ha = 10+D0;
    p = -2.;
    Rho = 1000.;
    printf("\n*** Z0=%1.2f\n", Z0);
    for (j=0; j<4; j++) {
      strcpy(Class, ca[j]);
      Ua = ua[j];
      Lm = 0.;
      Ust = 0.;
      Hm = 0.;
      ClcVdep();
      printf("*** Hm=%1.1f, Lm=%1.1f, Ua=%1.1f, Ha=%1.1f, Ust=%1.3f\n",
        Hm, Lm, Ua, Ha, Ust);
    }
    while (p <= 2.1) {
      D = pow(10., p)*1.e-6;
      printf("%7.3f  ", D*1.e6);
      for (j=0; j<4; j++) {
        strcpy(Class, ca[j]);
        Ua = ua[j];
        Lm = 0.;
        Ust = 0.;
        Hm = 0.;
        ClcVdep();
        printf("%7.4f %7.4f   ", Vd*100, Vdb*100);
      }
      printf("     %8.5f\n", Vs*100);
      p += .01;
    }
  }
}

// deposition velocity as function of u*
void dowork2() {
  int i;
  float da[8] = { 1, 3, 5, 10, 15, 30, 50, 100 };
  float us;
  Z0 = 0.1;
  D0 = 0.6;
  Zr = 1.;
  Rho = 1000.;
  for (i=0; i<8; i++) {
    D = da[i]*1.e-6;
    printf("\n*** D=%1.2f m\n", D*1.e6);
    for (us=0.1; us<=1.001; us+=0.01) {
      strcpy(Class, "III/1");
      Lm = 0;
      Hm = 0;
      Ua = 0;
      Ust = us;
      ClcVdep();
      printf("%8.4f %8.4f %8.4f\n", Ust*100, Vd*100, Vdb*100);
    }
    printf("\n");
  }
}

// scavenging rate
void dowork3(float iprec) {
  float p;
  p = -2.;
  printf("intensity (mm/h) %1.1f, Dw (m) %1.1f\n", iprec, L_PRT_DIA);
  printf("diameter (m) lambda*1.e6 (1/s)\n");
  printf("*\n");
  while (p <= 2.1) {
    D = pow(10., p)*1.e-6;
    printf("%7.3f  %1.2e\n", D*1.e6, ClcLambda(iprec)*1.e6);
    p += .01;
  }
}

// size distribution (Seinfeld, Table 7.3)
void dowork4() {
  int mode, ir;
  double m, n, s, nn[3], dd[3], ls[3];
  double p, d, f, ld, ldm, ldo;
  double intn, intm, intnmax, intmmax, d2n, d2m;
  
  // rural
  nn[0] = 6650;
  nn[1] = 147;
  nn[2] = 1990;
  dd[0] = 0.015;
  dd[1] = 0.054;
  dd[2] = 0.084;
  ls[0] = 0.225;
  ls[1] = 0.557;
  ls[2] = 0.266;
  
  // urban
  /* nn[0] = 99300;
  nn[1] = 1110;
  nn[2] = 36400;
  dd[0] = 0.013;
  dd[1] = 0.014;
  dd[2] = 0.05;
  ls[0] = 0.245;
  ls[1] = 0.666;
  ls[2] = 0.337;
  */
  
  intnmax = intmmax = 0;
  for (ir=0; ir<2; ir++) {
    p = -3;
    d2n = d2m = -1;
    intn = intm = 0;
    ld = p;
    while (p <= 2.1) {
      ldo = ld;
      ld = p;
      d = pow(10., p);
      m = 0.;
      s = 0.;
      n = 0.;
      for (mode=0; mode<3; mode++) {
        ldm = log10(dd[mode]);
        f = nn[mode]/(sqrt(2.*PI)*ls[mode]);
        f *= exp(-(ld-ldm)*(ld-ldm)/(2.*ls[mode]*ls[mode]));
        n += f;
        m += f * PI/6.* d*d*d;
        s += f * PI * d*d;
      }  
      p += 0.01;
      intn += (ld - ldo)*n;
      intm += (ld - ldo)*m;
      if (ir == 1) {
        printf("%8.4lf %10.2e %10.2e %10.2e\n", d, n, m, s);
        if (d2n < 0 && intn > intnmax/2.) d2n = d;
        if (d2m < 0 && intm > intmmax/2.) d2m = d;
      }
    }
    intnmax = intn;
    intmmax = intm;
  }
  printf("CMD=%e mum, MMD=%e mum\n", d2n, d2m);
}

// validation with data from Kfa Jlich (Jo 81)
void dowork5() {
  int i;
  double exp;
  double th[46];
  Zr = 0.8;
  Rho = 3606;
  Z0 = 0.05;
  D0 = 6*Z0;
  printf("@ exp    d    ust    vd_exp    vd_th  diff\n*\n");
  for (i=0; i<Nkfa; i++) {
    // Z0 = kfa_jo81[i][4]*0.01;
    D = kfa_jo81[i][2] * 1.e-6;
    Ust = kfa_jo81[i][3] * 0.01;
    exp = kfa_jo81[i][0];
    ClcVdep();
    th[i] = Vd*100;
    printf("@ %2d , %7.1f , %7.1f , %6.3f , %6.3f , %6.2f\n",
      (int)kfa_jo81[i][5], D*1.e6, Ust*100, exp, th[i], 2*(th[i]-exp)/(th[i]+exp) );
  }
  printf("\n\n");
  for (i=0; i<Nkfa; i++) {
    exp = kfa_jo81[i][0];
    printf("PU %6.3f %6.3f; PD %6.3f %6.3f;\n", 
      exp*(1-0.01*kfa_jo81[i][1]), th[i], exp*(1+0.01*kfa_jo81[i][1]), th[i]);
  }
}
